/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "can.h"

/******************************************************************************
* Macros
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/
const uint32_t CAN_ID[2] = { 0x0009FF37, 0x0019FF37 }; // 0 for Rx, 1 for Tx
const uint8_t DLC = 8;

/******************************************************************************
* Global Variables
******************************************************************************/
cmdecc_message_t CmdEccMsg;

/******************************************************************************
* External Variables
******************************************************************************/

/******************************************************************************
* Local Function Declarations
******************************************************************************/

/******************************************************************************
* Local Function Definitions
******************************************************************************/

/******************************************************************************
* Global Function Definitions
******************************************************************************/

/******************************************************************************
* Function:			void CAN_Init (void)
* Description:		CAN controller init
* Input:
* Output:
* Notes:
******************************************************************************/
void CAN_Init (void)
{
	uint8_t i;
	
	CANGCON = (1<<ENASTB); //Enable mode

	CANGIE = (1<<ENRX);	// Enable Receive interrupt
	CANIE2 = (1<<IEMOB0); // Only MOb0 interrupt is enabled


	// CAN baud rate = 100 kbit/s
	CANBT1 = 0x12;	// See datasheet for values (Table 19-2.)! Page 262
	CANBT2 = 0x04;
	CANBT3 = 0x13;

	for (i=0; i<15; i++)
	{
		CANPAGE = i << 4;	// Set initial value to all MOb registers
		CANSTMOB = 0;
		CANCDMOB = 0;
		CANIDT4 = 0;
		CANIDT3 = 0;
		CANIDT2 = 0;
		CANIDT1 = 0;
		CANIDM4 = 0;
		CANIDM3 = 0;
		CANIDM2 = 0;
		CANIDM1 = 0;
		CANSTMH = 0;
		CANSTML = 0;
	}
	
	CANGIE |= (1<<ENIT); // Enable all CAN interrupts
}

/******************************************************************************
* Function:			void CAN_ReceiveEnableMob0(void)
* Description:		Init and enable reception on MOb0
* Input:
* Output:
* Notes:
******************************************************************************/
void CAN_ReceiveEnableMob0(void)
{

	CANPAGE = 0;        // MOb0
	CANSTMOB = 0x00;	// Clear status flags

	CANIDM4 = 0xFF;		// Full filtering
	CANIDM3 = 0xFF;
	CANIDM2 = 0xFF;
	CANIDM1 = 0xFF;

	CANIDT4 = (CAN_ID[MSG_CMD_ECC] & 0x0000001F) << 3; // Receive only message with CAN_ID[0]
	CANIDT3 = CAN_ID[MSG_CMD_ECC] >> 5;
	CANIDT2 = CAN_ID[MSG_CMD_ECC] >> 13;
	CANIDT1 = CAN_ID[MSG_CMD_ECC] >> 21;

	CANCDMOB = (1 << CONMOB1) | (1<<IDE) | DLC; // Enable Rx, extended ID, set DLC to 8

}

/******************************************************************************
* Function:			void CAN_SendMob1 (const void* DataPtr)
* Description:		Transmit data frame on MOb1
* Input:			Pointer of the data bytes
* Output:
* Notes:
******************************************************************************/
void CAN_SendMob1 (const void* DataPtr)
{
	uint8_t i;

	CANPAGE = 1 << 4;   // MOb1
	CANSTMOB = 0x00;	// Clear status flags

	CANIDT4 = (CAN_ID[MSG_ST_ECC] & 0x0000001F) << 3; // Send CAN_ID[1]
	CANIDT3 = CAN_ID[MSG_ST_ECC] >> 5;
	CANIDT2 = CAN_ID[MSG_ST_ECC] >> 13;
	CANIDT1 = CAN_ID[MSG_ST_ECC] >> 21;
	CANCDMOB |= (1<<IDE); //Extended ID

	for (i = 0; i < DLC; i++)
	CANMSG = *((unsigned char*)DataPtr + i);
	
	CANCDMOB = (1 << CONMOB0) | DLC; // Enable Tx, set DLC to 8

}


/******************************************************************************
* Interrupt Routines
******************************************************************************/


/******************************************************************************
* Function:			ISR (CANIT_vect)
* Description:		CAN interrupt
* Input:		
* Output:
* Notes:		
******************************************************************************/
ISR (CANIT_vect)
{
	uint8_t i, dlc = 0;

	CANPAGE = 0;	// select MOb0, reset FIFO index

	if ( (CANSTMOB & (1<<RXOK)) != FALSE)	// Receive Complete
	{

		dlc = CANCDMOB & 0x0F;
		
		for (i=0; i<dlc; i++)
		  CmdEccMsg.BYTES[i] = CANMSG;
		
		CANSTMOB &= ~(1<<RXOK);	// clear RXOK flag
		CAN_ReceiveEnableMob0 ();	// enable next reception
	}
}
